import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.NoSuchElementException;
import java.util.Random;

public class AACTester {
	

	public static void main(String[] args) {
		System.out.println("Test AAC Category");
		testAACCat();
		System.out.println("Test AAC Mappings");
		testAACMapping("small.txt");
		System.out.println("on to medium");
		testAACMapping("medium.txt");
	}
	
	public static void testAACMapping(String filename) {
		AACMappings theirs = new AACMappings(filename);
		MyAACMappings mine = new MyAACMappings(filename);
		//System.out.println("here");
		compareStrings("constructor: getCurrCat",theirs.getCategory(),mine.getCategory());
		//System.out.println("there");
		testCompleteMapping(theirs,mine);
		//System.out.println("write");
		testWriteToFile("out"+filename,theirs, mine);
		//System.out.println("add");
		testAddItems(theirs,mine);
		//System.out.println("write2");
		testWriteToFile("outAdded"+filename,theirs, mine);
		try {
			theirs.reset();
			String s = theirs.select("empty.jpg");
			System.out.println("Did not error on non-exist image");
		}
		catch(Exception e) {
			System.out.println("Throw "+e.toString()+" on empty image");
		}
	}
	
	public static void testWriteToFile(String filename, AACMappings theirs, MyAACMappings mine) {
		theirs.writeToFile(filename);
		mine.writeToFile("my"+filename);
		AACMappings theirs2 = new AACMappings(filename);
		MyAACMappings mine2 = new MyAACMappings("my"+filename);
		System.out.println(filename+" written test");
		testCompleteMapping(theirs2,mine2);
	}
	
	
	public static void testCompleteMapping(AACMappings theirs, MyAACMappings mine) {
		theirs.reset();
		mine.reset();
		compareArrays(theirs.getImageLocs(), mine.getImageLocs());
		String catImg = theirs.getImageLocs()[0];
		String catImg2 = theirs.getImageLocs()[1];
		for(String catImage : mine.getImageLocs()) {
			//System.out.println("on cat "+catImage);
			theirs.reset();
			mine.reset();
			compareBooleans("hasImage",theirs.hasImage(catImage),mine.hasImage(catImage));
			testAACMapCat(catImage,theirs,mine);
		}
		theirs.reset();
		mine.reset();
		theirs.select(catImg);
		mine.select(catImg);
		try {
			String s = theirs.select(catImg2);
			System.out.println("They should have exception, returned "+s);
		}
		catch (NoSuchElementException e ){
			//do nothing
		}

	}
	
	public static void testAddItems(AACMappings theirs, MyAACMappings mine) {
		theirs.reset();
		mine.reset();
		theirs.addItem("DemoImage.jpg", "fake category");
		mine.addItem("DemoImage.jpg", "fake category");
		try {
			compareStrings("getText: in home after add",theirs.select("DemoImage.jpg"),mine.select("DemoImage.jpg"));
		}
		catch (NoSuchElementException e) {
			System.out.println("Threw no such as likely add updated cat");
		}
		compareStrings("after add & get cat: getCurrCat",theirs.getCategory(),mine.getCategory());
		theirs.addItem("DemoImage1.jpg", "fake item 1");
		mine.addItem("DemoImage1.jpg", "fake item 1");
		theirs.addItem("DemoImage2.jpg", "fake item 2");
		mine.addItem("DemoImage2.jpg", "fake item 2");
		theirs.addItem("DemoImage3.jpg", "fake item 3");
		mine.addItem("DemoImage3.jpg", "fake item 3");
		//System.out.println("Test mapping after adds");
		testCompleteMapping(theirs,mine);
	}
	
	public static void testAACMapCat(String catImage, AACMappings theirs, MyAACMappings mine) {
		theirs.reset();
		mine.reset();
		compareStrings("getText: in home",theirs.select(catImage),mine.select(catImage));
		compareStrings("in cat: getCurrCat",theirs.getCategory(),mine.getCategory());
		//System.out.println("Testing getImageLocs");
		compareArrays(theirs.getImageLocs(), mine.getImageLocs());
		String save = "";
		for(String itemImage : mine.getImageLocs()) {
			compareStrings("getText: in cat",theirs.select(itemImage),mine.select(itemImage));
			compareBooleans("hasImage",theirs.hasImage(itemImage),mine.hasImage(itemImage));
			save = itemImage;
			compareBooleans("hasImage - in cat check catImage",theirs.hasImage(catImage),mine.hasImage(catImage));

		}
		theirs.reset();
		mine.reset();
		compareBooleans("hasImage - in cate but now out",theirs.hasImage(save),mine.hasImage(save));

		
	}
	
	public static void testAACCat() {
		AACCategory theirs = new AACCategory("test");
		MyAACCategory mine = new MyAACCategory("test");
		testEmptyAACCat(theirs, mine);
		randomAddItem(5,theirs, mine);
		testFilledAACCat(theirs,mine);
		randomAddItem(10,theirs, mine);
		testFilledAACCat(theirs,mine);
	}
	
	public static void testEmptyAACCat(AACCategory theirs, MyAACCategory mine) {
		compareStrings("AACCat: getCat",theirs.getCategory(),mine.getCategory());
		compareBooleans("AACCat: hasImage",theirs.hasImage("tests.txt"),mine.hasImage("test.txt"));
		try {
			String s = theirs.select("empty.jpg");
			System.out.println("Did not error on non-exist image");
		}
		catch(Exception e) {
			System.out.println("They threw "+e.toString()+" on empty image");
		}
	}
	
	public static void testFilledAACCat(AACCategory theirs, MyAACCategory mine) {
		compareStrings("AACCat: getCat",theirs.getCategory(),mine.getCategory());
		compareBooleans("AACCat: hasImage",theirs.hasImage("tests.txt"),mine.hasImage("test.txt"));
		//System.out.println("Comparing images");
		compareArrays(theirs.getImageLocs(),mine.getImageLocs());
		for(String image : mine.getImageLocs()) {
			compareStrings("AACCat: getText",theirs.select(image),mine.select(image));
		}
	}
	
	public static void randomAddItem(int numToAdd, AACCategory theirs, MyAACCategory mine) {
		for(int i = 0; i<numToAdd; i++) {
			theirs.addItem("image"+i+".txt","text"+i);
			mine.addItem("image"+i+".txt","text"+i);
		}
	}
	
	public static void compareStrings(String method, String theirs, String mine) {
		if(!theirs.equalsIgnoreCase(mine)) {
			System.out.println("Error on method: "+method);
			System.out.println("Expected: '"+mine+"' Found: '"+theirs+"'");
		}
	}
	
	public static void compareBooleans(String method, boolean theirs, boolean mine) {
		if(theirs != mine) {
			System.out.println("Error on method: "+method);
			System.out.println("Expected: '"+mine+"' Found: '"+theirs+"'");
		}
	}
	
	public static void compareArrays(String[] theirs, String[] mine) {
		if(theirs.length != mine.length) {
			System.out.println("Error: Arrays different length");
		}
		//find missing
		for(String c: mine) {
			boolean found = false;
			for(String t : theirs) {
				if(c.equals(t)) {
					found = true;
					break;
				}
			}
			if(!found) {
				System.out.println("Error in getImages/getImageLocs: Theirs is missing "+c);
			}
		}
		
		//find extra
		for(String t : theirs) {
			boolean found = false;
			for(String c : mine) {
				if(c.equals(t)) {
					found = true;
					break;
				}
			}
			if(!found) {
				System.out.println("Error in getImages/getImageLocs: Theirs has extra "+t);
			}
		}
	}
	
	public static void generateAACMappingFile(String filename, int numCat, int maxItemsInCat) {
		try {
			FileWriter out = new FileWriter(new File(filename));
			Random r = new Random();
			for(int i = 0; i< numCat; i++) {
				String cat = "cat"+i;
				out.write(cat+".png"+" "+cat+"name"+i+"\n");
				int catItems = r.nextInt(maxItemsInCat);
				for(int j = 0; j<catItems; j++) {
					out.write(">"+cat+"image"+j+".png"+" "+cat+"item"+j+"\n");
				}
			}
			out.close();
			

			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
